/*
 * CPM (clock and power management) support for the X1600.
 *
 * Copyright (C) 2017, 2018, 2020, 2021, 2023,
 *               2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/devices/clocks.h>
#include <l4/sys/types.h>
#include <stdint.h>



/* X1600-specific clocks. */

enum Clock_x1600_identifiers
{
  Clock_x1600_identifier_base   = Clock_specific_start,
  Clock_usb_phy_12MHz           = Clock_specific_start,
  Clock_x1600_identifier_count, /* not a clock: limit for array definition */
};



#ifdef __cplusplus

#include <l4/devices/cpm-generic.h>

class Cpm_x1600_chip : public Cpm_chip
{
public:
  explicit Cpm_x1600_chip(l4_addr_t addr);
};

Cpm_chip *x1600_cpm_chip(l4_addr_t cpm_base);

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *x1600_cpm_init(l4_addr_t cpm_base);

const char *x1600_cpm_clock_type(void *cpm, enum Clock_identifiers clock);

int x1600_cpm_have_clock(void *cpm, enum Clock_identifiers clock);
void x1600_cpm_start_clock(void *cpm, enum Clock_identifiers clock);
void x1600_cpm_stop_clock(void *cpm, enum Clock_identifiers clock);

int x1600_cpm_get_parameters(void *cpm, enum Clock_identifiers clock,
                             uint32_t parameters[]);
int x1600_cpm_set_parameters(void *cpm, enum Clock_identifiers clock,
                             int num_parameters, uint32_t parameters[]);

uint8_t x1600_cpm_get_source(void *cpm, enum Clock_identifiers clock);
void x1600_cpm_set_source(void *cpm, enum Clock_identifiers clock, uint8_t source);
enum Clock_identifiers x1600_cpm_get_source_clock(void *cpm, enum Clock_identifiers clock);
void x1600_cpm_set_source_clock(void *cpm, enum Clock_identifiers clock, enum Clock_identifiers source);

uint64_t x1600_cpm_get_source_frequency(void *cpm, enum Clock_identifiers clock);

uint64_t x1600_cpm_get_frequency(void *cpm, enum Clock_identifiers clock);
int x1600_cpm_set_frequency(void *cpm, enum Clock_identifiers clock, uint64_t frequency);

EXTERN_C_END
